const fs = require('fs');
const path = require('path');
const mime = require('mime-types');
const { declare } = require('@babel/helper-plugin-utils');
const { types } = require('@babel/core');

/**
 * This is fork from
 * https://github.com/pomle/babel-plugin-inline-import-data-uri
 */
module.exports = declare((api) => {
    api.assertVersion(7);

    const visitor = {
        ImportDeclaration(path, state) {
            const src = path.node.source.value;
            const reference = state && state.file && state.file.opts.filename;
            const extensions = state && state.opts && state.opts.extensions;

            if (isInlined(src, extensions)) {
                if (path.node.specifiers.length > 1) {
                    throw new Error(
                        `Destructuring inlined import is not allowed. Check the import statement for '${src}'`
                    );
                }

                const id = path.node.specifiers[0].local.name;
                const data = toDataURI(src, reference);
                const variable = types.variableDeclarator(types.identifier(id), types.stringLiteral(data));

                path.replaceWith({
                    type: 'VariableDeclaration',
                    kind: 'const',
                    declarations: [variable],
                    leadingComments: [
                        {
                            type: 'CommentBlock',
                            value: `@fcc/babel-plugin-inline-import '${src}' `,
                        },
                    ],
                });
            }
        },
        CallExpression(path, state) {
            const callee = path.get('callee');
            const reference = state && state.file && state.file.opts.filename;
            const extensions = state && state.opts && state.opts.extensions;

            if (callee.isIdentifier() && callee.equals('name', 'require')) {
                const arg = path.get('arguments')[0];

                if (arg && arg.isStringLiteral()) {
                    const src = arg.node.value;

                    if (isInlined(src, extensions)) {
                        const data = toDataURI(src, reference);
                        path.replaceWith(types.stringLiteral(data));
                    }
                }
            }
        },
    };

    return {
        name: 'babel-plugin-inline-import',
        visitor,
    };
});

/**
 * Check inline file from extensions
 *
 * @param {string} src The source path
 * @param {Array} extensions The extensions list
 * @returns {boolean} The returns status
 */
function isInlined(src, extensions) {
    const accept = Array.isArray(extensions)
        ? extensions
        : ['.png', '.jpg', '.jpeg', '.svg', '.eot', '.svg', '.ttf', '.woff', 'woff2'];

    for (const extension of accept) {
        if (src.endsWith(extension)) {
            return true;
        }
    }

    return false;
}

/**
 * Convert file to base64
 *
 * @param {string} src The source path
 * @param {stirng} reference The target file proccesing
 * @returns {boolean} The returns convert string
 */
function toDataURI(src, reference) {
    src = src[0] === '.' ? path.resolve(path.dirname(reference), src) : '';

    const buffer = fs.readFileSync(require.resolve(src));
    const mimeType = mime.lookup(src) || 'application/octet-stream';

    return `data:${mimeType};base64,${buffer.toString('base64')}`;
}
